@@ -72,9 +72,9 @@ int two_colr_boxes = 0;
 int premultiplied_alpha = 0;
 const char* encoderId = nullptr;
 
-int nclx_matrix_coefficients = 6;
-int nclx_colour_primaries = 2;
-int nclx_transfer_characteristic = 2;
+uint16_t nclx_matrix_coefficients = 6;
+uint16_t nclx_colour_primaries = 2;
+uint16_t nclx_transfer_characteristic = 2;
 int nclx_full_range = true;
 
 const int OPTION_NCLX_MATRIX_COEFFICIENTS = 1000;
@@ -139,12 +139,12 @@ void show_help(const char* argv0)
             << "  --colour_primaries        nclx profile: color primaries (see h.273)\n"
             << "  --transfer_characteristic nclx profile: transfer characteristics (see h.273)\n"
             << "  --full_range_flag         nclx profile: full range flag, default: 1\n"
-            << "  --enable-two-colr-boxes   will write both an ICC and an nclx color profile if both a present\n"
+            << "  --enable-two-colr-boxes   will write both an ICC and an nclx color profile if both are present\n"
             << "  --premultiplied-alpha     input image has premultiplied alpha\n"
             << "\n"
             << "Note: to get lossless encoding, you need this set of options:\n"
             << "  -L                       switch encoder to lossless mode\n"
-            << "  -p chroma=444            switch off color subsampling\n"
+            << "  -p chroma=444            switch off chroma subsampling\n"
             << "  --matrix_coefficients=0  encode in RGB color-space\n";
 }
 
@@ -957,9 +957,9 @@ void list_encoder_parameters(heif_encoder* encoder)
 
         if (valid_options) {
           std::cerr << ", { ";
-          for (int i = 0; valid_options[i]; i++) {
-            if (i > 0) { std::cerr << ","; }
-            std::cerr << valid_options[i];
+          for (int k = 0; valid_options[k]; k++) {
+            if (k > 0) { std::cerr << ","; }
+            std::cerr << valid_options[k];
           }
           std::cerr << " }";
         }
@@ -972,9 +972,9 @@ void list_encoder_parameters(heif_encoder* encoder)
 }
 
 
-void set_params(struct heif_encoder* encoder, std::vector<std::string> params)
+void set_params(struct heif_encoder* encoder, const std::vector<std::string>& params)
 {
-  for (std::string p : params) {
+  for (const std::string& p : params) {
     auto pos = p.find_first_of('=');
     if (pos == std::string::npos || pos == 0 || pos == p.size() - 1) {
       std::cerr << "Encoder parameter must be in the format 'name=value'\n";
@@ -1006,8 +1006,18 @@ static void show_list_of_encoders(const heif_encoder_descriptor*const* encoder_d
 }
 
 
+class LibHeifInitializer {
+public:
+  LibHeifInitializer() { heif_init(nullptr); }
+  ~LibHeifInitializer() { heif_deinit(); }
+};
+
+
 int main(int argc, char** argv)
 {
+  // This takes care of initializing libheif and also deinitializing it at the end to free all resources.
+  LibHeifInitializer initializer;
+
   int quality = 50;
   bool lossless = false;
   std::string output_filename;
@@ -1065,13 +1075,13 @@ int main(int argc, char** argv)
         encoderId = optarg;
         break;
       case OPTION_NCLX_MATRIX_COEFFICIENTS:
-        nclx_matrix_coefficients = atoi(optarg);
+        nclx_matrix_coefficients = (uint16_t)strtoul(optarg, nullptr, 0);
         break;
       case OPTION_NCLX_COLOUR_PRIMARIES:
-        nclx_colour_primaries = atoi(optarg);
+        nclx_colour_primaries = (uint16_t)strtoul(optarg, nullptr, 0);
         break;
       case OPTION_NCLX_TRANSFER_CHARACTERISTIC:
-        nclx_transfer_characteristic = atoi(optarg);
+        nclx_transfer_characteristic = (uint16_t)strtoul(optarg, nullptr, 0);
         break;
       case OPTION_NCLX_FULL_RANGE_FLAG:
         nclx_full_range = atoi(optarg);
@@ -1118,6 +1128,7 @@ int main(int argc, char** argv)
     return 0;
   }
 
+  const heif_encoder_descriptor* active_encoder_descriptor = nullptr;
   if (count > 0) {
     int idx = 0;
     if (encoderId != nullptr) {
@@ -1140,6 +1151,8 @@ int main(int argc, char** argv)
       std::cerr << error.message << "\n";
       return 5;
     }
+
+    active_encoder_descriptor = encoder_descriptors[idx];
   }
   else {
     std::cerr << "No " << (enc_av1f ? "AV1" : "HEVC") << " encoder available.\n";
@@ -1212,19 +1225,38 @@ int main(int argc, char** argv)
     }
 
     heif_color_profile_nclx nclx;
-    nclx.matrix_coefficients = (heif_matrix_coefficients) nclx_matrix_coefficients;
-    nclx.transfer_characteristics = (heif_transfer_characteristics) nclx_transfer_characteristic;
-    nclx.color_primaries = (heif_color_primaries) nclx_colour_primaries;
+    error = heif_nclx_color_profile_set_matrix_coefficients(&nclx, nclx_matrix_coefficients);
+    if (error.code) {
+      std::cerr << "Invalid matrix coefficients specified.\n";
+      exit(5);
+    }
+    error = heif_nclx_color_profile_set_transfer_characteristics(&nclx, nclx_transfer_characteristic);
+    if (error.code) {
+      std::cerr << "Invalid transfer characteristics specified.\n";
+      exit(5);
+    }
+    error = heif_nclx_color_profile_set_color_primaries(&nclx, nclx_colour_primaries);
+    if (error.code) {
+      std::cerr << "Invalid color primaries specified.\n";
+      exit(5);
+    }
     nclx.full_range_flag = (uint8_t) nclx_full_range;
 
     //heif_image_set_nclx_color_profile(image.get(), &nclx);
 
+    if (lossless) {
+      if (heif_encoder_descriptor_supportes_lossless_compression(active_encoder_descriptor)) {
+        heif_encoder_set_lossless(encoder, lossless);
+      }
+      else {
+        std::cerr << "Warning: the selected encoder does not support lossless encoding. Encoding in lossy mode.\n";
+      }
+    }
+
     heif_encoder_set_lossy_quality(encoder, quality);
-    heif_encoder_set_lossless(encoder, lossless);
     heif_encoder_set_logging_level(encoder, logging_level);
 
     set_params(encoder, raw_params);
-
     struct heif_encoding_options* options = heif_encoding_options_alloc();
     options->save_alpha_channel = (uint8_t) master_alpha;
     options->save_two_colr_boxes_when_ICC_and_nclx_available = (uint8_t)two_colr_boxes;
